// SPDX-FileCopyrightText: Copyright 2015-2025 go-swagger maintainers
// SPDX-License-Identifier: Apache-2.0

// Package analysis contains integration tests for the go-openapi/analysis module.
package analysis
